// record.c (Musterloesung)
// Simple Adress-Datenbank
// Hans-Georg Esser, 29.04.2014

#include <string.h>  // strcpy, strcmp
#include <fcntl.h>   // open
#include <stdio.h>   // printf, scanf
#include <unistd.h>  // lseek
#include <sys/stat.h>    // S_IWUSR etc. 
 
typedef struct {
  int  id : 15;      // 15 Bits
  int  deleted : 1;  // 1 Bit
  char name[30];
  char vorname[30];
  char telefon[30];
  char gebdatum[11];
} record;

int recsize = sizeof(record);

int main () {
  record r;
  int fd;
  char filename[]="adressen.dat";
  char action[100];
  
  strncpy (r.name, "Mueller", 29);
  strncpy (r.vorname, "Anton", 29);
  strncpy (r.telefon, "0172/12345", 29);
  strncpy (r.gebdatum, "25.12.1980", 10);
  r.deleted = 0;

  int i;
  fd = open (filename, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);

  if ( lseek(fd,0,SEEK_END) == 0 ) {
    // Datei ist leer; Standardinhalte erzeugen
    printf ("open: Initialisiere Daten\n");
    for (i=0; i<100; i++) {
      r.id = i;
      write (fd, &r, recsize);
    }
  } else printf ("open: Daten existieren schon\n");

  printf ("Aktionen: show n, edit n, del n, new 0, search 0, quit 0\n");

  while (1) {
    printf ("Aktion Nummer: ");
    scanf ("%s %d", (char*)&action, &i);

    // action: show
    if (!strcmp((char*)&action, "show")) {
      lseek (fd, i*recsize, SEEK_SET);
      read (fd, &r, recsize);
      if (!r.deleted) {
        printf ("ID:   %d\n", r.id);
        printf ("Name: %s %s\n", r.vorname, r.name);
        printf ("Tel.: %s\n", r.telefon);
        printf ("Geb.: %s\n", r.gebdatum);
      } else printf ("Record %d deleted\n", i);
    };

    // action: edit
    if (!strcmp((char*)&action, "edit")) {
      lseek (fd, i*recsize, SEEK_SET);
      read (fd, &r, recsize);
      if (!r.deleted) { 
        printf ("ID: %d, Name: %s %s\n", r.id, r.vorname, r.name);
        printf ("Eingabe: Name Vorname Telefon Geburtstag: ");
        scanf ("%s %s %s %s", r.name, r.vorname, r.telefon, r.gebdatum);
        lseek (fd, i*recsize, SEEK_SET);
        write (fd, &r, recsize);
      } else printf ("Record %d deleted\n", i);
    };
    
    // action: delete
    if (!strcmp((char*)&action, "del")) {
      lseek (fd, i*recsize, SEEK_SET);
      read (fd, &r, recsize);
      r.deleted = 1;
      lseek (fd, i*recsize, SEEK_SET);
      write (fd, &r, recsize);
    };

    // action: new (Aufgabe 7 b)
    if (!strcmp((char*)&action, "new")) {
      lseek (fd, -recsize, SEEK_END);  // auf letzten Eintrag springen
      read (fd, &r, recsize);
      r.id++;                          // ID um 1 erhhen
      r.deleted = 0;                   // nicht gelscht
      printf ("Eingabe: Name Vorname Telefon Geburtstag: ");
      scanf ("%s %s %s %s", r.name, r.vorname, r.telefon, r.gebdatum);
      write (fd, &r, recsize);
    };
    
    // action: search (Aufgabe 7 c)
    if (!strcmp((char*)&action, "search")) {
      char search[100];
      printf ("Suchbegriff: ");
      scanf ("%s", (char*)&search);
      lseek (fd, 0, SEEK_SET);
      while (read (fd, &r, recsize)) {
        if ((r.deleted == 0) && 
            ((strcmp(r.vorname, search) == 0) ||
             (strcmp(r.name, search) == 0)))
          printf ("Treffer in record %d, %s %s\n", r.id, r.vorname, r.name);
          
      }
    }

    // action: quit
    if (!strcmp((char*)&action, "quit")) {
      printf ("Programmende.\n");
      break;
    };

  };

  close (fd);  
};
